/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoComposeContextMenuProvider.java,v 1.7 2004/05/17 14:02:54 dwagelaa Exp $
 * $Log: CoComposeContextMenuProvider.java,v $
 * Revision 1.7  2004/05/17 14:02:54  dwagelaa
 * + added initial ApplySolutionPattern functionality
 * 0 changed EPSGraphics to AWTGraphics
 * + added initial SVG export functionality (EPS works by now)
 * 0 updated version numbers to 0.2.11
 *
 * Revision 1.6  2004/05/11 08:18:44  dwagelaa
 * + added the ModelTransformation language element
 *
 * Revision 1.5  2004/04/30 16:24:55  dwagelaa
 * + added ElementConstraint support for the CoCompose2 editor
 *
 * Revision 1.4  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 * Revision 1.3  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.eclipse;

import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IWorkbenchActionConstants;

import be.ac.vub.cocompose.eclipse.actions.ApplySolutionPatternAction;
import be.ac.vub.cocompose.eclipse.actions.CreateElementConstraintAction;
import be.ac.vub.cocompose.eclipse.actions.CreateImplementationGeneratorAction;
import be.ac.vub.cocompose.eclipse.actions.CreateImplementationPatternAction;
import be.ac.vub.cocompose.eclipse.actions.CreateImplementationRolePartAction;
import be.ac.vub.cocompose.eclipse.actions.CreateModelTransformationAction;
import be.ac.vub.cocompose.eclipse.actions.CreateSimplePropertyAction;
import be.ac.vub.cocompose.eclipse.actions.CreateSolutionPatternAction;

/**
 * CoCompose context menu.
 * @author Dennis Wagelaar
 * @version 0.2.11, 17/05/2004
 * @since 0.2.0
 */
public class CoComposeContextMenuProvider extends ContextMenuProvider
{
  private ActionRegistry actionRegistry;

  /**
   * Creates a CoCompose context menu provider.
   * @param viewer
   * @param registry
   */
  public CoComposeContextMenuProvider(EditPartViewer viewer, ActionRegistry registry)
  {
    super(viewer);
    setActionRegistry(registry);
  }

  /**
   * @see org.eclipse.gef.ContextMenuProvider#buildContextMenu(org.eclipse.jface.action.IMenuManager)
   */
  public void buildContextMenu(IMenuManager manager)
  {
    // New Actions
    MenuManager newmenu = new MenuManager("&New");

    IAction action = getActionRegistry().getAction(CreateSolutionPatternAction.ID);
    if (action.isEnabled())
      newmenu.add(action);

    action = getActionRegistry().getAction(CreateModelTransformationAction.ID);
    if (action.isEnabled())
      newmenu.add(action);

    action = getActionRegistry().getAction(CreateImplementationPatternAction.ID);
    if (action.isEnabled())
      newmenu.add(action);

    action = getActionRegistry().getAction(CreateImplementationGeneratorAction.ID);
    if (action.isEnabled())
      newmenu.add(action);

    action = getActionRegistry().getAction(CreateImplementationRolePartAction.ID);
    if (action.isEnabled())
      newmenu.add(action);

    if (!newmenu.isEmpty())
      newmenu.add(new Separator());

    action = getActionRegistry().getAction(CreateSimplePropertyAction.ID);
    if (action.isEnabled())
      newmenu.add(action);

    action = getActionRegistry().getAction(CreateElementConstraintAction.ID);
    if (action.isEnabled())
      newmenu.add(action);

    if (!newmenu.isEmpty())
      manager.add(newmenu);

    // Refine Actions
    MenuManager refmenu = new MenuManager("&Refine");

    action = getActionRegistry().getAction(ApplySolutionPatternAction.ID);
    if (action.isEnabled())
      refmenu.add(action);

    if (!refmenu.isEmpty())
      manager.add(refmenu);

    // GEF crap...
    GEFActionConstants.addStandardActionGroups(manager);
    manager.add(new GroupMarker(GEFActionConstants.GROUP_FILE));

    action = getActionRegistry().getAction(GEFActionConstants.UNDO);
    manager.appendToGroup(GEFActionConstants.GROUP_UNDO, action);

    action = getActionRegistry().getAction(GEFActionConstants.REDO);
    manager.appendToGroup(GEFActionConstants.GROUP_UNDO, action);

    action = getActionRegistry().getAction(IWorkbenchActionConstants.COPY);
    if (action.isEnabled())
      manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);

    action = getActionRegistry().getAction(IWorkbenchActionConstants.PASTE);
    if (action.isEnabled())
      manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);

    action = getActionRegistry().getAction(IWorkbenchActionConstants.DELETE);
    if (action.isEnabled())
      manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);

    action = getActionRegistry().getAction(GEFActionConstants.DIRECT_EDIT);
    if (action.isEnabled())
      manager.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
      
    action = getActionRegistry().getAction(GEFActionConstants.EXPORT);
    if (action.isEnabled())
      manager.appendToGroup(GEFActionConstants.GROUP_FILE, action);

    // Alignment Actions
    MenuManager submenu = new MenuManager("&Align");

    action = getActionRegistry().getAction(GEFActionConstants.ALIGN_LEFT);
    if (action.isEnabled())
      submenu.add(action);

    action = getActionRegistry().getAction(GEFActionConstants.ALIGN_CENTER);
    if (action.isEnabled())
      submenu.add(action);

    action = getActionRegistry().getAction(GEFActionConstants.ALIGN_RIGHT);
    if (action.isEnabled())
      submenu.add(action);

    action = getActionRegistry().getAction(GEFActionConstants.ALIGN_TOP);
    if (action.isEnabled())
      submenu.add(action);

    action = getActionRegistry().getAction(GEFActionConstants.ALIGN_MIDDLE);
    if (action.isEnabled())
      submenu.add(action);

    action = getActionRegistry().getAction(GEFActionConstants.ALIGN_BOTTOM);
    if (action.isEnabled())
      submenu.add(action);
      
    if (!submenu.isEmpty())
      manager.appendToGroup(GEFActionConstants.GROUP_REST, submenu);
  }

  /**
   * Gets the action registry.
   * @return the action registry.
   */
  private ActionRegistry getActionRegistry()
  {
    return actionRegistry;
  }

  /**
   * Sets the action registry. 
   * @param registry the action registry.
   */
  private void setActionRegistry(ActionRegistry actionRegistry)
  {
    this.actionRegistry = actionRegistry;
  }
}
