/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoComposeEditor.java,v 1.14 2004/05/18 13:52:34 dwagelaa Exp $
 * $Log: CoComposeEditor.java,v $
 * Revision 1.14  2004/05/18 13:52:34  dwagelaa
 * version 0.2.11 release
 *
 * Revision 1.13  2004/05/18 11:40:13  dwagelaa
 * 0 worked on EPS and SVG export: both work now, except that images aren't supported in SVG
 * 0 worked on ApplySolutionPattern: the result is now properly displayed in the editor, and undoable ;-)
 *
 * Revision 1.12  2004/05/17 14:02:54  dwagelaa
 * + added initial ApplySolutionPattern functionality
 * 0 changed EPSGraphics to AWTGraphics
 * + added initial SVG export functionality (EPS works by now)
 * 0 updated version numbers to 0.2.11
 *
 * Revision 1.11  2004/05/11 08:18:44  dwagelaa
 * + added the ModelTransformation language element
 *
 * Revision 1.10  2004/05/07 14:19:28  dwagelaa
 * + added model pool (shared model) support
 * + added inter-model reference support
 *
 * Revision 1.9  2004/04/30 16:24:55  dwagelaa
 * + added ElementConstraint support for the CoCompose2 editor
 *
 * Revision 1.8  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 * Revision 1.7  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.eclipse;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.EventObject;

import org.eclipse.core.internal.runtime.Assert;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.dnd.TemplateTransferDragSourceListener;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.AlignmentAction;
import org.eclipse.gef.ui.actions.DirectEditAction;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.PrintAction;
import org.eclipse.gef.ui.actions.ZoomInAction;
import org.eclipse.gef.ui.actions.ZoomOutAction;
import org.eclipse.gef.ui.palette.PaletteContextMenuProvider;
import org.eclipse.gef.ui.palette.PaletteViewer;
import org.eclipse.gef.ui.parts.GraphicalEditorWithPalette;
import org.eclipse.gef.ui.parts.GraphicalViewerKeyHandler;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.gef.ui.parts.SelectionSynchronizer;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.gef.ui.stackview.CommandStackInspectorPage;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.actions.ApplySolutionPatternAction;
import be.ac.vub.cocompose.eclipse.actions.CoComposeCopyAction;
import be.ac.vub.cocompose.eclipse.actions.CoComposePasteAction;
import be.ac.vub.cocompose.eclipse.actions.CreateElementConstraintAction;
import be.ac.vub.cocompose.eclipse.actions.CreateImplementationGeneratorAction;
import be.ac.vub.cocompose.eclipse.actions.CreateImplementationPatternAction;
import be.ac.vub.cocompose.eclipse.actions.CreateImplementationRolePartAction;
import be.ac.vub.cocompose.eclipse.actions.CreateModelTransformationAction;
import be.ac.vub.cocompose.eclipse.actions.CreateSimplePropertyAction;
import be.ac.vub.cocompose.eclipse.actions.CreateSolutionPatternAction;
import be.ac.vub.cocompose.eclipse.actions.ExportAction;
import be.ac.vub.cocompose.eclipse.dnd.CoComposeTemplateTransferDropTargetListener;
import be.ac.vub.cocompose.eclipse.dnd.TextTransferDropTargetListener;
import be.ac.vub.cocompose.eclipse.editparts.GraphicalPartFactory;
import be.ac.vub.cocompose.eclipse.palette.CoComposePaletteCustomizer;
import be.ac.vub.cocompose.io.ModelReader;
import be.ac.vub.cocompose.io.ModelWriter;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.Model;

/**
 * Eclipse editor for CoCompose models.
 * @author Dennis Wagelaar
 * @version 0.2.11, 18/05/2004
 * @since 0.2.0
 */
public class CoComposeEditor extends GraphicalEditorWithPalette
{
  /** 
   * This class listens to changes to the file system in the workspace, and 
   * makes changes accordingly.
   * 1) An open, saved file gets deleted -> close the editor
   * 2) An open file gets renamed or moved -> change the editor's input accordingly
   * @author IBM Corporation
   * @see org.eclipse.gef.examples.logicdesigner.LogicEditor
   */
  class ResourceTracker implements IResourceChangeListener, IResourceDeltaVisitor
  {
    /**
     * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
     */
    public void resourceChanged(IResourceChangeEvent event)
    {
      IResourceDelta delta = event.getDelta();
      try
      {
        if (delta != null)
          delta.accept(this);
      }
      catch (CoreException exception)
      {
        // What should be done here?
      }
    }
    
    /**
     * @see org.eclipse.core.resources.IResourceDeltaVisitor#visit(org.eclipse.core.resources.IResourceDelta)
     */
    public boolean visit(IResourceDelta delta)
    {
      if (delta == null)
        return true;
      if (!delta.getResource().equals(
         ((FileEditorInput) getEditorInput()).getFile()))
        return true;
  
      if (delta.getKind() == IResourceDelta.REMOVED)
      {
        if ((IResourceDelta.MOVED_TO & delta.getFlags()) == 0)
        { // if the file was deleted
          // NOTE: The case where an open, unsaved file is deleted is being handled by the 
          // PartListener added to the Workbench in the initialize() method.
          if (!isDirty())
            closeEditor(false);
        }
        else
        { // else if it was moved or renamed
          final IFile newFile =
            ResourcesPlugin.getWorkspace().getRoot().getFile(
              delta.getMovedToPath());
          Display display = getSite().getShell().getDisplay();
          display.asyncExec(new Runnable()
          {
            public void run()
            {
              superSetInput(new FileEditorInput(newFile));
            }
          });
        }
      }
      return false;
    }
  }

  public static final int ID_MODEL = 1;

  protected static final String PALETTE_SIZE = "Palette Size";
  protected static final int DEFAULT_PALETTE_SIZE = 130;

  private PaletteRoot root = null;
  private KeyHandler sharedKeyHandler;
  private Model model = null;
  private ResourceTracker resourceListener = new ResourceTracker();
  private boolean savePreviouslyNeeded = false;
  private CoCompose settings = CoCompose.getDefault();

  /**
   * Listens for file system changes and acts upon file deletion.
   * @author IBM Corporation
   * @see org.eclipse.gef.examples.logicdesigner.LogicEditor
   */
  private IPartListener partListener = new IPartListener()
  {
      // If an open, unsaved file was deleted, query the user to either do a "Save As"
      // or close the editor.
    public void partActivated(IWorkbenchPart part)
    {
      if (part != CoComposeEditor.this)
        return;
      if (!((FileEditorInput) getEditorInput()).getFile().exists())
      {
        Shell shell = getSite().getShell();
        String title = "Unsaved file deleted";
        String message = 
          "An unsaved file was deleted, would you like to save it under another name or close the file?";
        String[] buttons =
        {
          "Save",
          "Close" 
        };
        MessageDialog dialog =
          new MessageDialog(
            shell,
            title,
            null,
            message,
            MessageDialog.QUESTION,
            buttons,
            0);
        if (dialog.open() == 0)
        {
          if (!performSaveAs())
            partActivated(part);
        }
        else
        {
          closeEditor(false);
        }
      }
    }
    public void partBroughtToTop(IWorkbenchPart part)
    {
    }
    public void partClosed(IWorkbenchPart part)
    {
    }
    public void partDeactivated(IWorkbenchPart part)
    {
    }
    public void partOpened(IWorkbenchPart part)
    {
    }
  };

  /**
   * Creates a CoComposeEditor. 
   */
  public CoComposeEditor()
  {
    setEditDomain(new DefaultEditDomain(this));
  }

  /**
   * Gets the CoCompose model that is edited.
   * @return the model.
   */
  public Model getModel()
  {
    return model;
  }

  /**
   * Sets the CoCompose model that is edited.
   * @param model - the model.
   */
  public void setModel(Model model)
  {
    if (this.model != null)
      settings.removeSharedModel(this.model);
    this.model = model;
    if (this.model != null)
      settings.addSharedModel(this.model);
    if ((getGraphicalViewer() != null) && (model != null))
      getGraphicalViewer().setContents(model);
    this.firePropertyChange(ID_MODEL);
  }

  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditorWithPalette#getPaletteRoot()
   */
  protected PaletteRoot getPaletteRoot()
  {
    if (root == null)
    {
      root = CoComposePlugin.createPalette();
    }
    return root;
  }

  /**
   * @see org.eclipse.ui.part.EditorPart#setInput(org.eclipse.ui.IEditorInput)
   */
  public void setInput(IEditorInput input)
  {
    super.setInput(input);
    
    Assert.isTrue(input instanceof IFileEditorInput);
    Model model = settings.getSharedModel(getUri());
    if (model == null)
    { // shared model not present
      IFile file = ((IFileEditorInput) input).getFile();
      try
      {
        ModelReader reader = (ModelReader) Class.forName(
          settings.getSetting("model.reader")).newInstance();
        model = reader.read(file.getContents(false), getUri(), settings.getFactory());
      }
      catch (Exception e) 
      {
        settings.getLog().report(e);
      }
    }
    if (model != null)
    {
      setModel(model);
      setTitle(model.getName());
    }
  }

  /**
   * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
   */
  public void doSave(IProgressMonitor monitor)
  {
    try
    {
      performSave(monitor);
      getCommandStack().markSaveLocation();
    }
    catch (Exception e)
    {
      settings.getLog().report(e);
    }
  }

  /**
   * @see org.eclipse.ui.ISaveablePart#doSaveAs()
   */
  public void doSaveAs()
  {
    performSaveAs();
  }

  /**
   * @see org.eclipse.ui.IEditorPart#gotoMarker(org.eclipse.core.resources.IMarker)
   */
  public void gotoMarker(IMarker marker)
  {
  }

  /**
   * @see org.eclipse.ui.ISaveablePart#isDirty()
   */
  public boolean isDirty()
  {
    return getCommandStack().isDirty();
  }

  /**
   * @see org.eclipse.ui.ISaveablePart#isSaveOnCloseNeeded()
   */
  public boolean isSaveOnCloseNeeded() 
  {
    return isDirty();
  }

  /**
   * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
   */
  public boolean isSaveAsAllowed()
  {
    return true;
  }

  /**
   * @see org.eclipse.gef.commands.CommandStackListener#commandStackChanged(java.util.EventObject)
   */
  public void commandStackChanged(EventObject event)
  {
    if (isDirty())
    {
      if (!isSavePreviouslyNeeded())
      {
        setSavePreviouslyNeeded(true);
        firePropertyChange(IEditorPart.PROP_DIRTY);
      }
    }
    else
    {
      setSavePreviouslyNeeded(false);
      firePropertyChange(IEditorPart.PROP_DIRTY);
    }
    super.commandStackChanged(event);
  }

  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditorWithPalette#configurePaletteViewer()
   */
  protected void configurePaletteViewer() 
  {
    super.configurePaletteViewer();
    PaletteViewer viewer = (PaletteViewer) getPaletteViewer();
    ContextMenuProvider provider = new PaletteContextMenuProvider(viewer);
    getPaletteViewer().setContextMenu(provider);
    viewer.setCustomizer(new CoComposePaletteCustomizer());
  }

  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditor#configureGraphicalViewer()
   */
  protected void configureGraphicalViewer()
  {
    super.configureGraphicalViewer();
    ScrollingGraphicalViewer viewer = (ScrollingGraphicalViewer) getGraphicalViewer();

    ScalableFreeformRootEditPart root = new ScalableFreeformRootEditPart();

    IAction zoomIn = new ZoomInAction(root.getZoomManager());
    IAction zoomOut = new ZoomOutAction(root.getZoomManager());
    getActionRegistry().registerAction(zoomIn);
    getActionRegistry().registerAction(zoomOut);
    getSite().getKeyBindingService().registerAction(zoomIn);
    getSite().getKeyBindingService().registerAction(zoomOut);

//  Printing is probably added automatically, but Eclipse just
//  won't print under Linux/GTK
    IAction print = new PrintAction(this);
    getActionRegistry().registerAction(print);
    getSite().getKeyBindingService().registerAction(print);
    
    viewer.setRootEditPart(root);

    viewer.setEditPartFactory(new GraphicalPartFactory());
    ContextMenuProvider provider = new CoComposeContextMenuProvider(viewer, getActionRegistry());
    viewer.setContextMenu(provider);
    getSite().registerContextMenu("be.ac.vub.cocompose.editor.contextmenu", //$NON-NLS-1$
      provider, viewer);
    viewer.setKeyHandler(new GraphicalViewerKeyHandler(viewer)
      .setParent(getCommonKeyHandler()));  
  }

  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditor#initializeGraphicalViewer()
   */
  protected void initializeGraphicalViewer()
  {
    getGraphicalViewer().setContents(getModel());
    getGraphicalViewer().addDropTargetListener(
      new CoComposeTemplateTransferDropTargetListener(getGraphicalViewer()));
    getGraphicalViewer().addDropTargetListener(
      new TextTransferDropTargetListener(
        getGraphicalViewer(),
        TextTransfer.getInstance()));
  }

  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditorWithPalette#initializePaletteViewer()
   */
  protected void initializePaletteViewer() 
  {
    super.initializePaletteViewer();
    getPaletteViewer().addDragSourceListener(
      new TemplateTransferDragSourceListener(getPaletteViewer()));
    CoComposePlugin.getDefault().getPreferenceStore().setDefault(
          PALETTE_SIZE, DEFAULT_PALETTE_SIZE);
  }

  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditor#createActions()
   */
  protected void createActions() 
  {
    super.createActions();
    ActionRegistry registry = getActionRegistry();
    IAction action;
  
    // Who the *($#^& copies stuff from the Palette anyway??
//    action = new CopyTemplateAction(this);
//    registry.registerAction(action);
//    getSelectionActions().add(action.getId());
//
//    action = new CoComposePasteTemplateAction(this);
//    registry.registerAction(action);
//    getSelectionActions().add(action.getId());

    action = new CoComposeCopyAction(this);
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new CoComposePasteAction(this);
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new ExportAction(this);
    getActionRegistry().registerAction(action);
    getSelectionActions().add(action.getId());

    action = new DirectEditAction((IWorkbenchPart) this);
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.LEFT);
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.RIGHT);
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.TOP);
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.BOTTOM);
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.CENTER);
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.MIDDLE);
    registry.registerAction(action);
    getSelectionActions().add(action.getId());
    
    action = new CreateSolutionPatternAction(this, settings.getFactory());
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new CreateImplementationPatternAction(this, settings.getFactory());
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new CreateImplementationGeneratorAction(this, settings.getFactory());
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new CreateImplementationRolePartAction(this, settings.getFactory());
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new CreateModelTransformationAction(this, settings.getFactory());
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new CreateSimplePropertyAction(this, settings.getFactory());
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new CreateElementConstraintAction(this, settings.getFactory());
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    action = new ApplySolutionPatternAction(this, settings.getFactory());
    registry.registerAction(action);
    getSelectionActions().add(action.getId());
  }

  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditorWithPalette#getInitialPaletteSize()
   */
  protected int getInitialPaletteSize() 
  {
    return CoComposePlugin.getDefault().getPreferenceStore().getInt(PALETTE_SIZE);
  }

  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditorWithPalette#handlePaletteResized(int)
   */
  protected void handlePaletteResized(int newSize) 
  {
    CoComposePlugin.getDefault().getPreferenceStore().setValue(PALETTE_SIZE, newSize);
  }
  
  /**
   * Extended to return ZoomManager on request.
   * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
   */
  public Object getAdapter(Class type)
  {
    if (type == CommandStackInspectorPage.class)
      return new CommandStackInspectorPage(getCommandStack());
    if (type == IContentOutlinePage.class)
      return new CoComposeOutlinePage(new TreeViewer(), this);
    if (type == ZoomManager.class)
      return (
        (ScalableFreeformRootEditPart) getGraphicalViewer().getRootEditPart())
        .getZoomManager();
    return super.getAdapter(type);
  }
  
  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditor#getActionRegistry()
   */
  public ActionRegistry getActionRegistry()
  {
    return super.getActionRegistry();
  }
  
  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditor#getEditDomain()
   */
  public DefaultEditDomain getEditDomain()
  {
    return super.getEditDomain();
  }

  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditor#getGraphicalViewer()
   */
  public GraphicalViewer getGraphicalViewer()
  {
    return super.getGraphicalViewer();
  }

  /**
   * @see org.eclipse.gef.ui.parts.GraphicalEditor#getSelectionSynchronizer()
   */
  public SelectionSynchronizer getSelectionSynchronizer()
  {
    return super.getSelectionSynchronizer();
  }

  /**
   * Returns the KeyHandler with common bindings for both the Outline and Graphical Views.
   * For example, delete is a common action.
   * @see org.eclipse.gef.examples.logicdesigner.LogicEditor#getCommonKeyHandler()
   */
  public KeyHandler getCommonKeyHandler()
  {
    if (sharedKeyHandler == null)
    {
      sharedKeyHandler = new KeyHandler();
      sharedKeyHandler.put(
        KeyStroke.getPressed(SWT.DEL, 127, 0),
        getActionRegistry().getAction(GEFActionConstants.DELETE));
      sharedKeyHandler.put(
        KeyStroke.getPressed(SWT.F2, 0),
        getActionRegistry().getAction(GEFActionConstants.DIRECT_EDIT));
    }
    return sharedKeyHandler;
  }

  /**
   * Executes a Save As.
   * @return True if the file got written.
   * @see org.eclipse.gef.examples.logicdesigner.LogicEditor#performSaveAs()
   */
  protected boolean performSaveAs() 
  {
    SaveAsDialog dialog = new SaveAsDialog(getSite().getWorkbenchWindow().getShell());
    dialog.setOriginalFile(((IFileEditorInput) getEditorInput()).getFile());
    dialog.open();
    IPath path = dialog.getResult();
    if (path == null)
      return false;
    // set new URI
    IWorkspace workspace = ResourcesPlugin.getWorkspace();
    final IFile file = workspace.getRoot().getFile(path);
    // build Save As operation
    WorkspaceModifyOperation op = new WorkspaceModifyOperation() 
    {
      public void execute(final IProgressMonitor monitor) throws CoreException 
      {
        try 
        {
          performSave(monitor);
        } 
        catch (Exception e) 
        {
          settings.getLog().report(e);
        }
      }
    };
    // execute Save As and reload file in editor
    try 
    {
      new ProgressMonitorDialog(getSite().getWorkbenchWindow().getShell()).run(false, true, op);
      setInput(new FileEditorInput((IFile) file));
      getCommandStack().markSaveLocation();
    } 
    catch (Exception e) 
    {
      settings.getLog().report(e);
    } 
    return true;
  }
  
  /**
   * Executes a Save of the model.
   * @param monitor the progress monitor.
   * @throws ClassNotFoundException if the ModelWriter could not be retrieved from the settings.
   * @throws IllegalAccessException if the ModelWriter class could not be accessed.
   * @throws InstantiationException if the ModelWriter class could not be instantiated.
   * @throws IOException if there was a file I/O error.
   * @throws CoreException if there was a file I/O error.
   * @throws ModelElementException if there was an error building the file content.
   */
  protected void performSave(IProgressMonitor monitor) throws ClassNotFoundException, 
    IllegalAccessException, InstantiationException, 
    IOException, CoreException, ModelElementException
  {
    ModelWriter writer = (ModelWriter) Class.forName(
      settings.getSetting("model.writer")).newInstance();
    ByteArrayOutputStream output = new ByteArrayOutputStream();
    writer.write(getModel(), output);
    ByteArrayInputStream input = new ByteArrayInputStream(output.toByteArray());
    IFile file = ((IFileEditorInput) getEditorInput()).getFile();
    if (file.exists())
      file.setContents(input, true, false, monitor);
    else
      file.create(input, true, monitor);
    output.close();
  }

  /**
   * Closes this editor.
   * @param save saves model if true.
   * @see org.eclipse.gef.examples.logicdesigner.LogicEditor#closeEditor(boolean)
   */
  protected void closeEditor(boolean save) 
  {
    getSite().getPage().closeEditor(this, save);
  }
  
  /**
   * Invokes super.setInput() and handles adding and removing the ResourceListener.
   * @param input the editor input.
   * @see org.eclipse.gef.examples.logicdesigner.LogicEditor#superSetInput(IEditorInput)
   */
  protected void superSetInput(IEditorInput input) 
  {
    // The workspace never changes for an editor.  So, removing and re-adding the 
    // resourceListener is not necessary.  But it is being done here for the sake
    // of proper implementation.  Plus, the resourceListener needs to be added 
    // to the workspace the first time around.
    if (getEditorInput() != null)
    {
      IFile file = ((FileEditorInput) getEditorInput()).getFile();
      file.getWorkspace().removeResourceChangeListener(resourceListener);
    }

    super.setInput(input);

    if (getEditorInput() != null)
    {
      IFile file = ((FileEditorInput) getEditorInput()).getFile();
      file.getWorkspace().addResourceChangeListener(resourceListener);
    }
  }

  /**
   * @see org.eclipse.ui.part.WorkbenchPart#setSite(org.eclipse.ui.IWorkbenchPartSite)
   */
  protected void setSite(IWorkbenchPartSite site)
  {
    super.setSite(site);
    getSite().getWorkbenchWindow().getPartService().addPartListener(partListener);
  }

  /**
   * Returns whether a save was needed before - isDirty().
   * @return true if a save was needed before.
   */
  public boolean isSavePreviouslyNeeded()
  {
    return savePreviouslyNeeded;
  }

  /**
   * Sets whether a save was needed before - isDirty().
   * @param savePrevouslyNeeded
   */
  public void setSavePreviouslyNeeded(boolean savePrevouslyNeeded)
  {
    this.savePreviouslyNeeded = savePrevouslyNeeded;
  }

  /**
   * Returns the URI of the file being edited.
   * Note that the URI is based on the virtual Eclipse workspace
   * and may not point to a real filesystem path.
   * @return the URI of the file being edited.
   */
  public String getUri()
  {
    IFile file = ((IFileEditorInput) getEditorInput()).getFile();
    return file.getLocation().toString();
  }
  
  /**
   * Frees shared model.
   * @see org.eclipse.ui.IWorkbenchPart#dispose()
   */
  public void dispose()
  {
    setModel(null);
  }
}
