/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoComposeElementFactory.java,v 1.3 2004/04/28 12:24:36 dwagelaa Exp $
 * $Log: CoComposeElementFactory.java,v $
 * Revision 1.3  2004/04/28 12:24:36  dwagelaa
 * worked on drag-n-drop
 * updated copy/paste
 * updated deep copy operation
 *
 * Revision 1.2  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.eclipse;

import org.eclipse.core.internal.runtime.Assert;
import org.eclipse.gef.requests.CreationFactory;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.VisitedElement;
import be.ac.vub.cocompose.log.Log;

/**
 * Eclipse/GEF creation factory for CoCompose elements.
 * @author Dennis Wagelaar
 * @version 0.2.9, 28/04/2004
 * @since 0.2.8
 */
public class CoComposeElementFactory implements CreationFactory
{
  private ModelFactory factory;
  private Object template;

  /**
   * Creates new CoComposeElementFactory.
   * @param factory the factory to create the actual elements.
   * @param template the template that defines which element to create.
   */
  public CoComposeElementFactory(ModelFactory factory, Object template)
  {
    Assert.isNotNull(factory);
    this.factory = factory;
    this.template = template;
  }

  /**
   * @see org.eclipse.gef.requests.CreationFactory#getNewObject()
   */
  public Object getNewObject()
  {
    if (template instanceof String)
    {
      if (TemplateConstants.TEMPLATE_CONCEPT.equals(template))
        return factory.createConcept();
      if (TemplateConstants.TEMPLATE_ROLE.equals(template))
        return factory.createRole();
    }
    else if (template instanceof VisitedElement)
    { // return same element, since it is already a usable instance.
      return template;
//      try
//      {
//        CopyElement copy = new CopyElement(factory);
//        return copy.copy((VisitedElement) template);
//      }
//      catch (ModelElementException e)
//      {
//        getLog().report(e);
//      }
//    }
//    else if (template instanceof List)
//    {
//      try
//      {
//        CopyElement copy = new CopyElement(factory);
//        return copy.copy((List) template);
//      }
//      catch (ModelElementException e)
//      {
//        getLog().report(e);
//      }
    }
    return null;
  }

  /**
   * @see org.eclipse.gef.requests.CreationFactory#getObjectType()
   */
  public Object getObjectType()
  {
    return template;
  }
  
  /**
   * Retrieves the logging object.
   * @return the logging object.
   */
  protected Log getLog()
  {
    return CoCompose.getDefault().getLog();
  }
}
