/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ApplySolutionPatternAction.java,v 1.2 2004/05/18 11:40:13 dwagelaa Exp $
 * $Log: ApplySolutionPatternAction.java,v $
 * Revision 1.2  2004/05/18 11:40:13  dwagelaa
 * 0 worked on EPS and SVG export: both work now, except that images aren't supported in SVG
 * 0 worked on ApplySolutionPattern: the result is now properly displayed in the editor, and undoable ;-)
 *
 * Revision 1.1  2004/05/17 14:02:54  dwagelaa
 * + added initial ApplySolutionPattern functionality
 * 0 changed EPSGraphics to AWTGraphics
 * + added initial SVG export functionality (EPS works by now)
 * 0 updated version numbers to 0.2.11
 *
 */

package be.ac.vub.cocompose.eclipse.actions;

import org.eclipse.core.internal.runtime.Assert;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import be.ac.vub.cocompose.eclipse.CoComposeEditor;
import be.ac.vub.cocompose.eclipse.CoComposeImages;
import be.ac.vub.cocompose.eclipse.commands.ApplySolutionPatternCommand;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.core.SolutionPattern;

/**
 * Applies a solution pattern in the current model.
 * @author Dennis Wagelaar
 * @version 0.2.11, 17/05/2004
 * @since 0.2.11
 */
public class ApplySolutionPatternAction extends SelectionAction
{
  public static final String ID = "ApplySolutionPattern";

  private CoComposeEditor editor;
  private ModelFactory factory;
  private SolutionPattern solutionPattern = null;

  /**
   * Creates an ApplySolutionPatternAction.
   * @param editor the CoCompose editor.
   * @param factory the model element factory.
   */
  public ApplySolutionPatternAction(CoComposeEditor editor, ModelFactory factory)
  {
    super((IWorkbenchPart) editor);
    Assert.isNotNull(editor);
    Assert.isNotNull(factory);
    this.editor = editor;
    this.factory = factory;
    setText("Apply solution pattern");
    setId(ID);
    setToolTipText("Applies the selected solution pattern");
    setImageDescriptor(CoComposeImages.SOL_PATT_OBJ);
  }

  /**
   * @see org.eclipse.jface.action.IAction#run()
   */
  public void run()
  {
    execute(getCommand());
  }

  /**
   * Gets the selected solution pattern.
   * @return the selected solution pattern.
   */
  public SolutionPattern getSolutionPattern()
  {
    return solutionPattern;
  }

  /**
   * Sets the selected solution pattern.
   * @param pattern the selected solution pattern.
   */
  protected void setSolutionPattern(SolutionPattern pattern)
  {
    solutionPattern = pattern;
  }

  /**
   * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
   */
  protected boolean calculateEnabled()
  {
    return (getSolutionPattern() != null);
  }

  /**
   * Creates the command to be executed.
   * @return the command.
   */
  protected Command getCommand()
  {
    EditPart part = (EditPart) getSelectedObjects().get(0);
    SolutionPattern pattern = (SolutionPattern) part.getModel();
    ApplySolutionPatternCommand cmd = new ApplySolutionPatternCommand(editor, factory);
    cmd.setSolutionPattern(getSolutionPattern());
    return cmd;
  }

  /**
   * @see org.eclipse.gef.ui.actions.SelectionAction#handleSelectionChanged()
   */
  protected void handleSelectionChanged()
  {
    setSolutionPattern(null);
    ISelection selection = getSelection();
    if (!(selection instanceof IStructuredSelection))
      return;
    IStructuredSelection sel = (IStructuredSelection) selection;
    if (sel.size() != 1)
      return;
    Object obj = sel.getFirstElement();
    if (obj instanceof EditPart)
    {
      Object model = ((EditPart) obj).getModel();
      if (model instanceof SolutionPattern)
      {
        setSolutionPattern((SolutionPattern) model);
      }
    }
  }
}
