/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CreateElementConstraintAction.java,v 1.2 2004/05/10 14:26:23 dwagelaa Exp $
 * $Log: CreateElementConstraintAction.java,v $
 * Revision 1.2  2004/05/10 14:26:23  dwagelaa
 * worked on shared models and eclipse property sources
 *
 * Revision 1.1  2004/04/30 16:24:55  dwagelaa
 * + added ElementConstraint support for the CoCompose2 editor
 *
 */

package be.ac.vub.cocompose.eclipse.actions;

import org.eclipse.gef.commands.Command;
import org.eclipse.ui.IEditorPart;

import be.ac.vub.cocompose.eclipse.CoComposeImages;
import be.ac.vub.cocompose.eclipse.commands.CreateCommand;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.core.Namespace;

/**
 * Creates an ElementConstraint for the selected element.
 * @author Dennis Wagelaar
 * @version 0.2.10, 10/05/2004
 * @since 0.2.10
 */
public class CreateElementConstraintAction extends CreateAction
{
  public static final String ID = "CreateElementConstraint";
  
  /**
   * Creates a CreateElementConstraintAction.
   * @param editor the CoCompose editor.
   * @param factory the model element factory.
   */
  public CreateElementConstraintAction(IEditorPart editor, ModelFactory factory)
  {
    super(editor, factory, Namespace.class);
    setText("Element constraint");
    setId(ID);
    setToolTipText("Creates an element constraint within the selected element");
    setImageDescriptor(CoComposeImages.CONSTRAINT_OBJ);
  }

  /**
   * Creates the command to be executed.
   * @return the command.
   */
  protected Command getCommand()
  {
    CreateCommand cmd = (CreateCommand) super.getCommand();
    cmd.setElement(factory.createElementConstraint());
    return cmd;
  }
}
