/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ExportAction.java,v 1.4 2004/05/18 13:17:40 dwagelaa Exp $
 * $Log: ExportAction.java,v $
 * Revision 1.4  2004/05/18 13:17:40  dwagelaa
 * version 0.2.11 release
 *
 * Revision 1.3  2004/05/18 11:40:13  dwagelaa
 * 0 worked on EPS and SVG export: both work now, except that images aren't supported in SVG
 * 0 worked on ApplySolutionPattern: the result is now properly displayed in the editor, and undoable ;-)
 *
 * Revision 1.2  2004/05/17 14:02:54  dwagelaa
 * + added initial ApplySolutionPattern functionality
 * 0 changed EPSGraphics to AWTGraphics
 * + added initial SVG export functionality (EPS works by now)
 * 0 updated version numbers to 0.2.11
 *
 * Revision 1.1  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 */

package be.ac.vub.cocompose.eclipse.actions;

import java.awt.Canvas;
import java.awt.Graphics2D;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.jibble.epsgraphics.EpsGraphics2D;

import cwi.SVGGraphics.SVGGraphics;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.CoComposeEditor;
import be.ac.vub.cocompose.eclipse.draw2d.AWTGraphics;
import be.ac.vub.cocompose.eclipse.editparts.GraphicalPartFactory;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.log.Log;

/**
 * Exports the current model in another file format. Currently supports
 * EPS and SVG.
 * @author Dennis Wagelaar
 * @version 0.2.11, 18/05/2004
 * @since 0.2.9
 */
public class ExportAction extends SelectionAction
{
  private Log log = CoCompose.getDefault().getLog();
  private Object contents = null;
  private IFile file = null;
  
  /**
   * Creates a CoComposePasteAction.
   * @param part the CoCompose editor.
   */
  public ExportAction(IWorkbenchPart part)
  {
    super(part);
  }

  /**
   * @see org.eclipse.jface.action.IAction#run()
   */
  public void run()
  {
    CoComposeEditor editor = (CoComposeEditor) getWorkbenchPart();
    SaveAsDialog dialog = new SaveAsDialog(editor.getSite().getWorkbenchWindow().getShell());
    file = ((IFileEditorInput) editor.getEditorInput()).getFile();
    String filename = stripFilename(file);
    dialog.setOriginalName(filename);
    dialog.open();
    IPath path = dialog.getResult();
    if (path == null)
      return;
    // set new URI
    IWorkspace workspace = ResourcesPlugin.getWorkspace();
    file = workspace.getRoot().getFile(path);
    // build Save As operation
    WorkspaceModifyOperation op;
    if (file.getName().endsWith(".eps"))
    {
      op = buildEPSOperation();
    }
    else if (file.getName().endsWith(".svg"))
    {
      op = buildSVGOperation();
    }
    else
    {
      log.report(new IOException("Don't know how to export to " + 
        file.getName() + "; currently only EPS and SVG formats are supported."));
      return;
    }
    // execute Save As and reload file in editor
    try 
    {
      new ProgressMonitorDialog(
        editor.getSite().getWorkbenchWindow().getShell()).run(false, true, op);
      getCommandStack().markSaveLocation();
    } 
    catch (Exception e) 
    {
      log.report(e);
    } 
  }

  /**
   * Performs saving an EPS file.
   * @param monitor
   */
  public void performSaveEPS(IProgressMonitor monitor)
  {
    IFile file = getFile();
    String name = stripFilename(file);
    if (name.indexOf('.') > -1)
      name = name.substring(0, name.indexOf('.'));
    try
    {
      AbstractGraphicalEditPart part =
        (AbstractGraphicalEditPart) getRootEditPart();
      Graphics2D epsg = new EpsGraphics2D(name);
      AWTGraphics g = new AWTGraphics(epsg);
      g.setFontSizeDelta(1.2f);
      part.getFigure().paint(g);
      char[] eps = g.toString().toCharArray();
      byte[] beps = new byte[eps.length];
      for (int i = 0; i < eps.length; i++)
      {
        beps[i] = (byte) eps[i];
      }
      ByteArrayInputStream input = new ByteArrayInputStream(beps);
      if (file.exists())
        file.setContents(input, true, false, monitor);
      else
        file.create(input, true, monitor);
      g.dispose();
    }
    catch (Exception e)
    {
      log.report(e);
    }
  }

  /**
   * Performs saving an SVG file.
   * @param monitor
   */
  public void performSaveSVG(IProgressMonitor monitor)
  {
    IFile file = getFile();
    String name = stripFilename(file);
    try
    {
      AbstractGraphicalEditPart part =
        (AbstractGraphicalEditPart) getRootEditPart();
      Rectangle b = part.getContentPane().getClientArea();
      Canvas component = new Canvas();
      component.setBounds(b.x, b.y, b.width, b.height);
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      PrintStream ps = new PrintStream(out);
      SVGGraphics svgg = SVGGraphics.create(ps, component);
      AWTGraphics g = new AWTGraphics(svgg);
      part.getFigure().paint(g);
      svgg.close();
      ByteArrayInputStream input = new ByteArrayInputStream(out.toByteArray());
      if (file.exists())
        file.setContents(input, true, false, monitor);
      else
        file.create(input, true, monitor);
      g.dispose();
    }
    catch (Exception e)
    {
      log.report(e);
    }
  }

  /**
   * Retrieves the root editpart of the current model.
   * @return the root editpart of the current model.
   */
  protected RootEditPart getRootEditPart()
  {
    Shell shell = new Shell();
    GraphicalViewer viewer = new ScrollingGraphicalViewer();
    viewer.createControl(shell);
    viewer.setEditDomain(new DefaultEditDomain(null));
    viewer.setRootEditPart(new ScalableFreeformRootEditPart());
    viewer.setEditPartFactory(new GraphicalPartFactory());
    viewer.setContents(getContents());
    viewer.flush();
    return viewer.getRootEditPart();
  }

  /**
   * @see org.eclipse.gef.ui.actions.SelectionAction#handleSelectionChanged()
   */
  protected void handleSelectionChanged()
  {
    ISelection selection = getSelection();
    if (!(selection instanceof IStructuredSelection))
      return;
    IStructuredSelection sel = (IStructuredSelection) selection;
    Object obj = sel.getFirstElement();
    if (obj instanceof EditPart)
    {
      Object model = ((EditPart) obj).getModel();
      if (model instanceof ModelElement)
      {
        if (((ModelElement) model).getModel() != null)
          setContents(((ModelElement) model).getModel());
      }
    }
  }

  /**
   * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
   */
  protected boolean calculateEnabled()
  {
    return (getWorkbenchPart() instanceof CoComposeEditor)
        && (getContents() != null);
  }
  
  /**
   * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
   */
  protected void init()
  {
    setId(GEFActionConstants.EXPORT);
    setText("&Export...");
    setToolTipText("Export to external format...");
  }
  
  /**
   * Gets the file to which the EPS is exported.
   * @return the file to which the EPS is exported.
   */
  public IFile getFile()
  {
    return file;
  }
  
  /**
   * Gets the filename stripped from its extension.
   * @param file the file to get the name for.
   * @return the filename stripped from its extension.
   */
  protected String stripFilename(IFile file)
  {
    String name = file.getName();
    if (name.indexOf('.') > -1)
      name = name.substring(0, name.indexOf('.'));
    return name;
  }

  /**
   * Gets printing contents.
   * @return printing contents.
   */
  protected Object getContents() 
  {
    return contents;
  }

  /**
   * Sets printing contents.
   * @param o printing contents.
   */
  protected void setContents(Object o) 
  {
    contents = o;
  }

  /**
   * Builds a WorkspaceModifyOperation that generates an EPS file.
   * @return the WorkspaceModifyOperation.
   */  
  private WorkspaceModifyOperation buildEPSOperation()
  {
    WorkspaceModifyOperation op = new WorkspaceModifyOperation() 
    {
      public void execute(final IProgressMonitor monitor) throws CoreException 
      {
        try 
        {
          performSaveEPS(monitor);
        } 
        catch (Exception e)
        {
          log.report(e);
        }
      }
    };
    return op;
  }

  /**
   * Builds a WorkspaceModifyOperation that generates an SVG file.
   * @return the WorkspaceModifyOperation.
   */  
  private WorkspaceModifyOperation buildSVGOperation()
  {
    WorkspaceModifyOperation op = new WorkspaceModifyOperation() 
    {
      public void execute(final IProgressMonitor monitor) throws CoreException 
      {
        try 
        {
          performSaveSVG(monitor);
        } 
        catch (Exception e)
        {
          log.report(e);
        }
      }
    };
    return op;
  }
}
