/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.actions;

import org.eclipse.draw2d.PrintFigureOperation;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

/**
 * CoCompose printing dialog.
 * @author Dennis Wagelaar
 * @version 0.2.8, 20/04/2004
 * @since 0.2.8
 */
public class PrintModeDialog extends Dialog
{
  private Button tile;
  private Button fitPage;
  private Button fitWidth;
  private Button fitHeight;
  
  /**
   * Creates a PrintModeDialog.
   * @param parentShell
   */
  public PrintModeDialog(Shell parentShell)
  {
    super(parentShell);
  }

  /**
   * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
   */
  protected void cancelPressed() 
  {
    setReturnCode(-1);
    close();
  }

  /**
   * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
   */
  protected void configureShell(Shell newShell) 
  {
    newShell.setText("Select Print Mode");
    super.configureShell(newShell);
  }

  /**
   * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
   */
  protected Control createDialogArea(Composite parent) 
  {
    Composite composite = (Composite) super.createDialogArea(parent);
  
    tile = new Button(composite, SWT.RADIO);
    tile.setText("Tile");
    tile.setSelection(true);
  
    fitPage = new Button(composite, SWT.RADIO);
    fitPage.setText("Fit Page");

    fitWidth = new Button(composite, SWT.RADIO);
    fitWidth.setText("Fit Width");

    fitHeight = new Button(composite, SWT.RADIO);
    fitHeight.setText("Fit Height");

    return composite;
  }

  /**
   * @see org.eclipse.jface.dialogs.Dialog#okPressed()
   */
  protected void okPressed() 
  {
    int returnCode = -1;
    if (tile.getSelection())
      returnCode = PrintFigureOperation.TILE;
    else if (fitPage.getSelection())
      returnCode = PrintFigureOperation.FIT_PAGE;
    else if (fitHeight.getSelection())
      returnCode = PrintFigureOperation.FIT_HEIGHT;
    else if (fitWidth.getSelection())
      returnCode = PrintFigureOperation.FIT_WIDTH;
    setReturnCode(returnCode);
    close();
  }
}
