/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ApplySolutionPatternCommand.java,v 1.1 2004/05/17 14:02:54 dwagelaa Exp $
 * $Log: ApplySolutionPatternCommand.java,v $
 * Revision 1.1  2004/05/17 14:02:54  dwagelaa
 * + added initial ApplySolutionPattern functionality
 * 0 changed EPSGraphics to AWTGraphics
 * + added initial SVG export functionality (EPS works by now)
 * 0 updated version numbers to 0.2.11
 *
 */

package be.ac.vub.cocompose.eclipse.commands;

import org.eclipse.core.internal.runtime.Assert;
import org.eclipse.gef.commands.Command;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.CoComposeEditor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.SolutionPattern;
import be.ac.vub.cocompose.log.Log;
import be.ac.vub.cocompose.operations.ApplySolutionPattern;
import be.ac.vub.cocompose.operations.CopyElement;

/**
 * Eclipse GEF command for applying a solution pattern.
 * @author Dennis Wagelaar
 * @version 0.2.11, 17/05/2004
 * @since 0.2.11
 */
public class ApplySolutionPatternCommand extends Command
{
  private Log log = CoCompose.getDefault().getLog();
  private CoComposeEditor editor;
  private ModelFactory factory;
  private ApplySolutionPattern apply;
  private CopyElement copy;
  private SolutionPattern solutionPattern = null;
  private Model oldModel = null;

  /**
   * Creates an ApplySolutionPatternCommand.
   * @param editor the CoCompose editor.
   * @param factory the factory to create new elements.
   */
  public ApplySolutionPatternCommand(CoComposeEditor editor, ModelFactory factory)
  {
    super("Apply solution pattern");
    Assert.isNotNull(editor);
    this.editor = editor;
    this.factory = factory;
    this.apply = new ApplySolutionPattern(factory);
    this.copy = new CopyElement(factory);
  }

  /**
   * Gets the solution pattern to apply.
   * @return the solution pattern to apply.
   */
  public SolutionPattern getSolutionPattern()
  {
    return solutionPattern;
  }

  /**
   * Sets the solution pattern to apply.
   * @param pattern the solution pattern to apply.
   */
  public void setSolutionPattern(SolutionPattern pattern)
  {
    solutionPattern = pattern;
  }

  /**
   * @see org.eclipse.gef.commands.Command#execute()
   */
  public void execute() 
  {
    primExecute();
  }

  /**
   * @see org.eclipse.gef.commands.Command#redo()
   */
  public void redo() 
  {
    primExecute();
  }

  /**
   * @see org.eclipse.gef.commands.Command#undo()
   */
  public void undo() 
  {
    editor.setModel(oldModel);
  }

  /**
   * Execute implementation.
   * @see org.eclipse.gef.commands.Command#execute()
   */
  protected void primExecute()
  {
    Assert.isNotNull(solutionPattern);
    oldModel = solutionPattern.getModel();
    Assert.isNotNull(oldModel);
    try
    {
      Model targetModel = (Model) copy.copy(oldModel);
      copy.reset();
      SolutionPattern localPattern =
        (SolutionPattern) targetModel.getOwnedIds().get(solutionPattern.getId());
      apply.apply(localPattern);
      editor.setModel(targetModel);
    }
    catch (ModelElementException e)
    {
      log.report(e);
    }
  }
}
