/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: EPSGraphicsDebug.java,v 1.4 2004/05/17 14:02:54 dwagelaa Exp $
 * $Log: EPSGraphicsDebug.java,v $
 * Revision 1.4  2004/05/17 14:02:54  dwagelaa
 * + added initial ApplySolutionPattern functionality
 * 0 changed EPSGraphics to AWTGraphics
 * + added initial SVG export functionality (EPS works by now)
 * 0 updated version numbers to 0.2.11
 *
 * Revision 1.3  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 * Revision 1.2  2004/04/26 16:32:38  dwagelaa
 * Added CVS tags
 *
 */

package be.ac.vub.cocompose.eclipse.draw2d.debug;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.draw2d.EPSGraphics;
import be.ac.vub.cocompose.log.Log;

/**
 * Debugging code for EPSGraphics
 * @author Dennis Wagelaar
 * @version 0.2.11, 17/05/2004
 * @since 0.2.9
 * @see be.ac.vub.cocompose.eclipse.draw2d.EPSGraphics
 */
public aspect EPSGraphicsDebug
{
  private static Log log = CoCompose.getDefault().getLog();
  
  pointcut create(EPSGraphics g):
    target(g) &&
    initialization(EPSGraphics.new(..));

  pointcut dispose(EPSGraphics g):
    target(g) &&
    execution(void EPSGraphics.dispose());
    
  pointcut eachmethod(): execution(public * EPSGraphics.*(..));
  
  before(EPSGraphics g): create(g)
  {
    beforeCreation(g);
  }
  
  after(EPSGraphics g): dispose(g)
  {
    afterDispose();
  }
  
  before(): eachmethod()
  {
    String signature = thisJoinPointStaticPart.getSignature().toString();
    Object[] margs = thisJoinPoint.getArgs();
    for (int index = 0; index < margs.length; index++)
    {
      signature = signature + ", \"" + margs[index].toString() + "\"";
    }
    beforeTrace(signature);
  }

  after(): eachmethod()
  {
    afterTrace();
  }

  /**
   * Performs logging at creation of g.
   * @param g the graphics context.
   */
  public static void beforeCreation(EPSGraphics g)
  {
    log.log("Writing to EPS", Log.INFO, null);
  }

  /**
   * Closes off the logging.
   */  
  public static void afterDispose()
  {
    log.endLog(Log.INFO);
  }

  /**
   * Logs each method call.
   * @param sign the method signature
   */
  public static void beforeTrace(String sign)
  {
    log.log(sign + " invoked", Log.INFO, null);
  }

  /**
   * Ends log of each method call.
   */
  public static void afterTrace()
  {
    log.endLog(Log.INFO);
  }
}
