/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoComposeContainerTreeEditPart.java,v 1.2 2004/04/28 12:33:20 dwagelaa Exp $
 * $Log: CoComposeContainerTreeEditPart.java,v $
 * Revision 1.2  2004/04/28 12:33:20  dwagelaa
 * tree drag-n-drop works
 *
 */

package be.ac.vub.cocompose.eclipse.editparts;

import java.util.List;

import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;

import be.ac.vub.cocompose.eclipse.editpolicies.CoComposeContainerEditPolicy;
import be.ac.vub.cocompose.eclipse.editpolicies.CoComposeTreeContainerEditPolicy;
import be.ac.vub.cocompose.lang.core.Namespace;

/**
 * Eclipse tree editpart for CoCompose namespace elements (containers).
 * @author Dennis Wagelaar
 * @version 0.2.9, 28/04/2004
 * @since 0.2.0
 */
public class CoComposeContainerTreeEditPart extends CoComposeTreeEditPart
{
  /**
   * Creates a CoComposeContainerTreeEditPart.
   * @param model the model to edit.
   */
  public CoComposeContainerTreeEditPart(Object model)
  {
    super(model);
  }
  
  /**
   * Gets the model of this as a Namespace.
   * @return Model of this.
   */
  protected Namespace getNamespace() 
  {
    return (Namespace) getModel();
  }

  /**
   * Creates and installs edit policies.
   */
  protected void createEditPolicies() 
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.CONTAINER_ROLE, new CoComposeContainerEditPolicy());
    installEditPolicy(EditPolicy.TREE_CONTAINER_ROLE, new CoComposeTreeContainerEditPolicy());
    //If this editpart is the contents of the viewer, then it is not deletable!
    if (getParent() instanceof RootEditPart)
      installEditPolicy(EditPolicy.COMPONENT_ROLE, new RootComponentEditPolicy());
  }

  /**
   * Returns the children of this from the model.
   * @return  List of children.
   */
  protected List getModelChildren() 
  {
    return getNamespace().getOwnedElements();
  }
}
