/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.figures;

import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Manages the layout of concept elements.
 * @author Dennis Wagelaar
 * @version 0.2.0, 09/01/2004
 * @since 0.2.0
 */
public class ConceptLayout extends FreeformLayout
{
  private final int CHILD_TOP = 0;
  private final int CHILD_LEFT = 0;

  /**
   * Clips the Rectangle constraint to behave like a vertical flow layout.
   * @see org.eclipse.draw2d.LayoutManager#setConstraint(IFigure, Object)
   */
  public void setConstraint(IFigure figure, Object newConstraint) 
  {
    if (newConstraint instanceof Rectangle)
      clipConstraint(figure, (Rectangle) newConstraint);
    super.setConstraint(figure, newConstraint);
  }

  /**
   * Clips the position constraint to behave like a vertical flow layout.
   * @param figure - the figure of which to clip the position.
   * @param constraint - the rectangle to clip.
   */
  public void clipConstraint(IFigure figure, Rectangle constraint)
  {
    constraint.setLocation(Math.max(CHILD_LEFT, constraint.x), 
                           Math.max(CHILD_TOP, constraint.y));
  }
  
  /**
   * Forces expansion of preferred size by (1, 1) to prevent
   * clipping of contained figures.
   * @see org.eclipse.draw2d.AbstractLayout#calculatePreferredSize(org.eclipse.draw2d.IFigure, int, int)
   */
  protected Dimension calculatePreferredSize(IFigure f, int wHint, int hHint)
  {
    Dimension size = super.calculatePreferredSize(f, wHint, hHint);
    size = size.expand(1, 1);
    return size;
  }
}
