/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ConstrainedElementsPropertySource.java,v 1.2 2004/05/07 14:19:28 dwagelaa Exp $
 * $Log: ConstrainedElementsPropertySource.java,v $
 * Revision 1.2  2004/05/07 14:19:28  dwagelaa
 * + added model pool (shared model) support
 * + added inter-model reference support
 *
 * Revision 1.1  2004/04/30 16:24:55  dwagelaa
 * + added ElementConstraint support for the CoCompose2 editor
 *
 */

package be.ac.vub.cocompose.eclipse.model.datatypes;

import be.ac.vub.cocompose.lang.constraints.ElementConstraint;
import be.ac.vub.cocompose.lang.core.ModelElement;

/**
 * Property source for the constrainedElements property of ElementConstraint.
 * @author Dennis Wagelaar
 * @version 0.2.10, 30/04/2004
 * @since 0.2.10
 * @see be.ac.vub.cocompose.eclipse.model.constraints.EclipseElementConstraint
 */
public class ConstrainedElementsPropertySource
  extends ModelElementsPropertySource
{
  private ElementConstraint subject = null;
  
  /**
   * Creates a new ConstrainedElementsPropertySource.
   * @param subject the ElementConstraint for which to edit the property.
   */
  public ConstrainedElementsPropertySource(ElementConstraint subject)
  {
    super(subject, subject.getConstrainedElements(), subject.getRootNamespace(), null);
    this.subject = subject;
  }

  /**
   * @see be.ac.vub.cocompose.eclipse.model.datatypes.ModelElementsPropertySource#removeElement(be.ac.vub.cocompose.lang.core.ModelElement)
   */
  protected void removeElement(ModelElement element)
  {
    super.removeElement(element);
    subject.removeConstrainedElement(element);
  }

  /**
   * @see be.ac.vub.cocompose.eclipse.model.datatypes.ModelElementsPropertySource#addElement(int, be.ac.vub.cocompose.lang.core.ModelElement)
   */
  protected void addElement(int index, ModelElement element)
  {
    subject.addConstrainedElement(index, element);
    super.addElement(index, element);
  }
}
