/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ModelReader.java,v 1.4 2004/05/07 14:19:28 dwagelaa Exp $
 * $Log: ModelReader.java,v $
 * Revision 1.4  2004/05/07 14:19:28  dwagelaa
 * + added model pool (shared model) support
 * + added inter-model reference support
 *
 * Revision 1.3  2004/04/28 12:23:29  dwagelaa
 * + added parsing of ImplementationPattern.content tag
 *
 */

package be.ac.vub.cocompose.io;

import java.io.IOException;
import java.io.InputStream;

import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.ModelElementException;

/**
 * Generic model reading interface for reading a model from its URI.
 * @author Dennis Wagelaar
 * @version 0.2.10, 07/05/2004
 * @since 0.1.0
 */
public interface ModelReader
{
  /**
   * Reads the model from its uri.
   * @param uri the uri to read from.
   * @param factory the model factory used to create the elements.
   * @return the model
   * @exception IOException if the model could not be read.
   * @exception ModelElementException if a model element could not be read.
   */
  public Model read(String uri, ModelFactory factory) throws IOException, ModelElementException;

  /**
   * Reads the model from its uri.
   * @param input the stream to read from.
   * @param uri the uri of the model.
   * @param factory the model factory used to create the elements.
   * @return the model
   * @exception IOException if the model could not be read.
   * @exception ModelElementException if a model element could not be read.
   */
  public Model read(InputStream input, String uri, ModelFactory factory) throws IOException, ModelElementException;
}
