/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: XMIElementAttributesReader.java,v 1.3 2004/04/28 12:23:29 dwagelaa Exp $
 * $Log: XMIElementAttributesReader.java,v $
 * Revision 1.3  2004/04/28 12:23:29  dwagelaa
 * + added parsing of ImplementationPattern.content tag
 *
 */

package be.ac.vub.cocompose.io.xmi;

import org.xml.sax.Attributes;

import be.ac.vub.cocompose.lang.DefaultSuperModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.constraints.Constraint;
import be.ac.vub.cocompose.lang.core.ImplementationPattern;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.RefinedElement;

/**
 * Reads the XMI attributes of CoCompose model elements
 * @author Dennis Wagelaar
 * @version 0.2.3, 20/01/2004
 * @since 0.2.0
 */
public class XMIElementAttributesReader extends DefaultSuperModelVisitor
{
  private Attributes atts = null;
  
  /**
   * Sets the parsed XML attributes
   * @param atts the parsed XML attributes
   */
  public void setAttributes(Attributes atts)
  {
    this.atts = atts;
  }
  
  /**
   * Visits a ModelElement.
   * @param modelElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitModelElement(ModelElement modelElement) throws ModelElementException
  {
    super.visitModelElement(modelElement);
    String id = atts.getValue(XMI.qName("id"));
    if (id != null)
      modelElement.setId(id);
    String name = atts.getValue(XMI.CoCompose.qName("name"));
    if (name != null)
      modelElement.setName(name);
  }

  /**
   * Visits a RefinedElement.
   * @param refinedElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRefinedElement(RefinedElement refinedElement) throws ModelElementException
  {
    super.visitRefinedElement(refinedElement);
    String iconUri = atts.getValue(XMI.CoCompose.qName("iconUri"));
    if (iconUri != null)
      refinedElement.setIconUri(iconUri);
  }

  /**
   * Visits an ImplementationPattern.
   * @param implementationPattern - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationPattern(ImplementationPattern implementationPattern) throws ModelElementException
  {
    super.visitImplementationPattern(implementationPattern);
    String content = atts.getValue(XMI.CoCompose.qName("content"));
    if (content != null)
      implementationPattern.setContent(content);
  }

  /**
   * Visits a Constraint.
   * @param constraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitConstraint(Constraint constraint) throws ModelElementException
  {
    super.visitConstraint(constraint);
    String inverted = atts.getValue(XMI.CoCompose.qName("inverted"));
    if (inverted != null)
    {
      if (inverted.equals("true"))
        constraint.setInverted(true);
      else
        constraint.setInverted(false);
    }
  }
}
