/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: XMIElementLinkAttributesReader.java,v 1.5 2004/05/07 14:19:28 dwagelaa Exp $
 * $Log: XMIElementLinkAttributesReader.java,v $
 * Revision 1.5  2004/05/07 14:19:28  dwagelaa
 * + added model pool (shared model) support
 * + added inter-model reference support
 *
 * Revision 1.4  2004/04/28 12:23:29  dwagelaa
 * + added parsing of ImplementationPattern.content tag
 *
 * Revision 1.3  2004/04/26 17:39:41  dwagelaa
 * Changed the defaultConcept property into the defaultElement property, which is a RefinedElement.
 *
 */

package be.ac.vub.cocompose.io.xmi;

import java.util.Map;

import org.xml.sax.Attributes;

import be.ac.vub.cocompose.lang.DefaultSuperModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.RefinedElement;

/**
 * Reads the XMI linked attributes of CoCompose model elements
 * @author Dennis Wagelaar
 * @version 0.2.10, 07/05/2004
 * @since 0.2.3
 */
public class XMIElementLinkAttributesReader extends DefaultSuperModelVisitor
{
  private Attributes atts = null;
  private Map elementMap = null;
  
  /**
   * Creates an XMIModelLinkReader
   * @param elementMap the map of id -&gt; element
   */
  public XMIElementLinkAttributesReader(Map elementMap)
    throws ModelElementException
  {
    if (elementMap == null)
      throw new ModelElementException("ElementMap may not be null");
    this.elementMap = elementMap;
  }

  /**
   * Sets the parsed XML attributes
   * @param atts the parsed XML attributes
   */
  public void setAttributes(Attributes atts)
  {
    this.atts = atts;
  }

  /**
   * Visits a Model.
   * @param model the element to visit.
   * @exception ModelElementException
   */
  public void visitModel(Model model) throws ModelElementException
  {
    super.visitModel(model);
    String defElementId = atts.getValue(XMI.CoCompose.qName("defaultElement"));
    if (defElementId != null)
    {
      Object defElement = elementMap.get(defElementId);
      if (defElement instanceof RefinedElement)
      {
        model.setDefaultElement((RefinedElement) defElement);
        ((RefinedElement) defElement).setDefaultElementOf(model);
      }
    }
  }
}
