/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: XMIModelReaderDebug.java,v 1.2 2004/05/07 14:19:28 dwagelaa Exp $
 * $Log: XMIModelReaderDebug.java,v $
 * Revision 1.2  2004/05/07 14:19:28  dwagelaa
 * + added model pool (shared model) support
 * + added inter-model reference support
 *
 */

package be.ac.vub.cocompose.io.xmi.debug;

import org.xml.sax.Attributes;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.io.ModelReader;
import be.ac.vub.cocompose.io.xmi.XMIModelLinkReader;
import be.ac.vub.cocompose.io.xmi.XMIModelReader;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.log.Log;

/**
 * Debugging code for XMIModelReader.
 * @author Dennis Wagelaar
 * @version 0.2.10, 06/05/2004
 * @since 0.2.7
 * @see be.ac.vub.cocompose.io.xmi.XMIModelReader
 */
public aspect XMIModelReaderDebug
{
  private static Log log = CoCompose.getDefault().getLog();
  
  after(XMIModelReader r):
    target(r) &&
    execution(* startOwnedElement(*))
  {
    afterStartOwnedElement(r.getNamespace());
  }
  
  after(XMIModelReader r): 
    target(r) &&
    execution(* endOwnedElement())
  {
    afterEndOwnedElement(r.getNamespace());
  }
  
  after(XMIModelLinkReader r):
    target(r) &&
    execution(* startOwnedElement(*))
  {
    afterStartOwnedElement(r.getNamespace());
  }
  
  after(XMIModelLinkReader r): 
    target(r) &&
    execution(* endOwnedElement())
  {
    afterEndOwnedElement(r.getNamespace());
  }
  
  before(Attributes attrs):
    args(attrs) &&
    execution(* startMultiplicity(Attributes))
  {
    beforeStartMultiplicity(attrs);
  }
  
  Model around(String uri, ModelFactory factory):
    args(uri, factory) &&
    call(Model ModelReader.read(String, ModelFactory)) &&
    withincode(* XMIModelLinkReader.findHref(String))
  {
    Model model = proceed(uri, factory);
    afterReadModel(model);
    return model;
  }
  
  before(String id):
    args(id) &&
    call(* get(Object)) &&
    withincode(* XMIModelLinkReader.findHref(String))
  {
    beforeFindHrefGetFromId(id);
  }
  
  /**
   * Logs a message after a startOwnedElement().
   * @param ns The namespace after starting the new element.
   */
  public static void afterStartOwnedElement(Namespace ns)
  { 
    log.log("Namespace now is " + ns, Log.INFO, null);
    log.endLog(Log.INFO);
  }

  /**
   * Logs a message after an endOwnedElement().
   * @param ns The namespace after ending the element.
   */
  public static void afterEndOwnedElement(Namespace ns)
  { 
    log.log("Namespace re-set to " + ns, Log.INFO, null);
    log.endLog(Log.INFO);
  }
  
  /**
   * Logs a message of the multiplicity attributes before startMultiplicity().
   * @param attrs the XML attributes.
   */
  public static void beforeStartMultiplicity(Attributes attrs)
  {
    log.log("Reading multiplicity [" + attrs.getValue("lower") + ".." + 
      attrs.getValue("upper") + "]", Log.INFO, null);
    log.endLog(Log.INFO);
  }
  
  /**
   * Logs a message after reading a model.
   * @param model the model that was read.
   */
  public static void afterReadModel(Model model)
  {
    log.log("Read model " + model, Log.INFO, null);
    log.endLog(Log.INFO);
  }
  
  /**
   * Logs a message before getting an element by id.
   * @param id the element id.
   */
  public static void beforeFindHrefGetFromId(String id)
  {
    log.log("Getting element with id " + id, Log.INFO, null);
    log.endLog(Log.INFO);
  }
}
