/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: DefaultModelVisitor.java,v 1.3 2004/05/10 15:13:53 dwagelaa Exp $
 * $Log: DefaultModelVisitor.java,v $
 * Revision 1.3  2004/05/10 15:13:53  dwagelaa
 * + added the ModelTransformation language element
 * + added some more CVS Id tags
 * 0 refactored some classes to use Log
 *
 */

package be.ac.vub.cocompose.lang;

import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.ModelTransformation;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Refinement;
import be.ac.vub.cocompose.lang.core.SolutionPattern;
import be.ac.vub.cocompose.lang.core.ImplementationGenerator;
import be.ac.vub.cocompose.lang.core.ImplementationRolePart;
import be.ac.vub.cocompose.lang.core.ImplementationPattern;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.constraints.Constraint;
import be.ac.vub.cocompose.lang.constraints.ElementConstraint;
import be.ac.vub.cocompose.lang.constraints.CompositeConstraint;
import be.ac.vub.cocompose.lang.constraints.AndConstraint;
import be.ac.vub.cocompose.lang.constraints.OrConstraint;
import be.ac.vub.cocompose.lang.properties.Property;
import be.ac.vub.cocompose.lang.properties.SimpleProperty;
import be.ac.vub.cocompose.lang.properties.ModelProperty;
import be.ac.vub.cocompose.lang.datatypes.Multiplicity;

/**
 * Default implementation for visiting a Model.
 * @author Dennis Wagelaar 
 * @version 0.2.10, 10/05/2004
 * @since 0.2.0
 */
abstract public class DefaultModelVisitor implements ModelVisitor
{

  /**
   * The current visitor to accept, which can be either this or the stacked visitor.
   */
  protected ModelVisitor current = this;
  private ModelVisitor stacked = null;

  /**
   * Starts the visitor.
   * @param element - the element to start visiting at.
   * @exception ModelElementException
   */
  public void visit(VisitedElement element) throws ModelElementException
  {
    element.accept(current);
  }

  /**
   * Visits a ModelElement.
   * @param modelElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitModelElement(ModelElement modelElement) throws ModelElementException
  {
  }

  /**
   * Visits a Namespace.
   * @param namespace - the element to visit.
   * @exception ModelElementException
   */
  public void visitNamespace(Namespace namespace) throws ModelElementException
  {
  }

  /**
   * Visits a Model.
   * @param model - the element to visit.
   * @exception ModelElementException
   */
  public void visitModel(Model model) throws ModelElementException
  {
  }

  /**
   * Visits a Relationship.
   * @param relationship - the element to visit.
   * @exception ModelElementException
   */
  public void visitRelationship(Relationship relationship) throws ModelElementException
  {
  }

  /**
   * Visits a RelationElement.
   * @param relationElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRelationElement(RelationElement relationElement) throws ModelElementException
  {
  }

  /**
   * Visits a Refinement.
   * @param refinement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRefinement(Refinement refinement) throws ModelElementException
  {
  }

  /**
   * Visits a SolutionPattern.
   * @param solutionPattern - the element to visit.
   * @exception ModelElementException
   */
  public void visitSolutionPattern(SolutionPattern solutionPattern) throws ModelElementException
  {
  }

  /**
   * Visits an ImplementationGenerator.
   * @param implementationGenerator - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationGenerator(ImplementationGenerator implementationGenerator) throws ModelElementException
  {
  }

  /**
   * Visits an ImplementationRolePart.
   * @param rolePart - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationRolePart(ImplementationRolePart rolePart) throws ModelElementException
  {
  }

  /**
   * Visits an ImplementationPattern.
   * @param implementationPattern - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationPattern(ImplementationPattern implementationPattern) throws ModelElementException
  {
  }

  /**
   * Visits a ModelTransformation.
   * @param modelTransformation - the element to visit.
   * @exception ModelElementException
   */
  public void visitModelTransformation(ModelTransformation modelTransformation) throws ModelElementException
  {
  }

  /**
   * Visits a RefinedElement.
   * @param refinedElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRefinedElement(RefinedElement refinedElement) throws ModelElementException
  {
  }

  /**
   * Visits a concept.
   * @param concept - the element to visit.
   * @exception ModelElementException
   */
  public void visitConcept(Concept concept) throws ModelElementException
  {
  }

  /**
   * Visits a Role.
   * @param role - the element to visit.
   * @exception ModelElementException
   */
  public void visitRole(Role role) throws ModelElementException
  {
  }

  /**
   * Visits a Constraint.
   * @param constraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitConstraint(Constraint constraint) throws ModelElementException
  {
  }

  /**
   * Visits an ElementConstraint.
   * @param elementConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitElementConstraint(ElementConstraint elementConstraint) throws ModelElementException
  {
  }

  /**
   * Visits a CompositeConstraint.
   * @param compositeConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitCompositeConstraint(CompositeConstraint compositeConstraint) throws ModelElementException
  {
  }

  /**
   * Visits an AndConstraint.
   * @param andConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitAndConstraint(AndConstraint andConstraint) throws ModelElementException
  {
  }

  /**
   * Visits an OrConstraint.
   * @param orConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitOrConstraint(OrConstraint orConstraint) throws ModelElementException
  {
  }

  /**
   * Visits a Property.
   * @param property - the element to visit.
   * @exception ModelElementException
   */
  public void visitProperty(Property property) throws ModelElementException
  {
  }

  /**
   * Visits a SimpleProperty.
   * @param simpleProperty - the element to visit.
   * @exception ModelElementException
   */
  public void visitSimpleProperty(SimpleProperty simpleProperty) throws ModelElementException
  {
  }

  /**
   * Visits a ModelProperty.
   * @param modelProperty - the element to visit.
   * @exception ModelElementException
   */
  public void visitModelProperty(ModelProperty modelProperty) throws ModelElementException
  {
  }

  /**
   * Visits a Multiplicity.
   * @param multiplicity - the element to visit.
   * @exception ModelElementException
   */
  public void visitMultiplicity(Multiplicity multiplicity) throws ModelElementException
  {
  }

  /**
   * Gets the stacked visitor. @return the stacked visitor. 
   */
  public ModelVisitor getStacked()
  {
    return stacked;
  }

  /**
   * Sets a stacked visitor. @param stacked - the stacked visitor. 
   */
  public void setStacked(ModelVisitor stacked)
  {
    this.stacked = stacked;
  }

}
