/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ModelFactory.java,v 1.2 2004/05/10 15:13:53 dwagelaa Exp $
 * $Log: ModelFactory.java,v $
 * Revision 1.2  2004/05/10 15:13:53  dwagelaa
 * + added the ModelTransformation language element
 * + added some more CVS Id tags
 * 0 refactored some classes to use Log
 *
 */

package be.ac.vub.cocompose.lang;

import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelTransformation;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.SolutionPattern;
import be.ac.vub.cocompose.lang.core.ImplementationGenerator;
import be.ac.vub.cocompose.lang.core.ImplementationRolePart;
import be.ac.vub.cocompose.lang.core.ImplementationPattern;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.constraints.ElementConstraint;
import be.ac.vub.cocompose.lang.constraints.AndConstraint;
import be.ac.vub.cocompose.lang.constraints.OrConstraint;
import be.ac.vub.cocompose.lang.properties.SimpleProperty;
import be.ac.vub.cocompose.lang.properties.ModelProperty;

/**
 * Abstract factory interface for creating model elements.
 * @author Dennis Wagelaar 
 * @version 0.2.10, 10/05/2004
 * @since 0.2.0
 */
public interface ModelFactory
{
  /**
   * Creates a Model.
   * @return the Model.
   */
  public Model createModel();

  /**
   * Creates a Relationship.
   * @return the Relationship.
   */
  public Relationship createRelationship();

  /**
   * Creates a SolutionPattern.
   * @return the SolutionPattern.
   */
  public SolutionPattern createSolutionPattern();

  /**
   * Creates an ImplementationGenerator.
   * @return the ImplementationGenerator.
   */
  public ImplementationGenerator createImplementationGenerator();

  /**
   * Creates an ImplementationRolePart.
   * @return the ImplementationRolePart.
   */
  public ImplementationRolePart createImplementationRolePart();

  /**
   * Creates an ImplementationPattern.
   * @return the ImplementationPattern.
   */
  public ImplementationPattern createImplementationPattern();

  /**
   * Creates a ModelTransformation.
   * @return the ModelTransformation.
   */
  public ModelTransformation createModelTransformation();

  /**
   * Creates a Concept.
   * @return the Concept.
   */
  public Concept createConcept();

  /**
   * Creates a Role.
   * @return the Role.*/
  public Role createRole();

  /**
   * Creates an ElementConstraint.
   * @return the ElementConstraint.*/
  public ElementConstraint createElementConstraint();

  /**
   * Creates an AndConstraint.
   * @return the AndConstraint.*/
  public AndConstraint createAndConstraint();

  /**
   * Creates an OrConstraint.
   * @return the OrConstraint.*/
  public OrConstraint createOrConstraint();

  /**
   * Creates a SimpleProperty.
   * @return the SimpleProperty.*/
  public SimpleProperty createSimpleProperty();

  /**
   * Creates a ModelProperty.
   * @return the ModelProperty.*/
  public ModelProperty createModelProperty();
}
