/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: RefinedElement.java,v 1.4 2004/05/10 14:26:23 dwagelaa Exp $
 * $Log: RefinedElement.java,v $
 * Revision 1.4  2004/05/10 14:26:23  dwagelaa
 * worked on shared models and eclipse property sources
 *
 * Revision 1.3  2004/04/26 17:39:41  dwagelaa
 * Changed the defaultConcept property into the defaultElement property, which is a RefinedElement.
 *
 */

package be.ac.vub.cocompose.lang.core;

import java.util.List;

/**
 * Interface for elements that can have refinements. The "CompositeType" construct was previously used for this.
 * @author Dennis Wagelaar
 * @version 0.2.10, 10/05/2004
 * @since 0.1.0 
 */
public interface RefinedElement extends RelationElement
{
  /**
   * Returns the refinements for this stereotype.
   * @return a copy of the List of refinements.
   */
  public List getRefinements();

  /**
   * Gets the inherited elements.
   * @return the list of inherited elements.
   */
  public List getInheritsFrom();


  /**
   * Adds an inherited element to this concept.
   * @param inheritsFrom the inherited element to add.
   */
  public void addInheritsFrom(RefinedElement inheritsFrom);

  /**
   * Adds an inherited element to this concept.
   * @param index the index at which to add the element.
   * @param inheritsFrom the inherited element to add.
   */
  public void addInheritsFrom(int index, RefinedElement inheritsFrom);

  /**
   * Removes an inherited element from this concept.
   * @param inheritsFrom the inherited element to remove.
   */
  public void removeInheritsFrom(RefinedElement inheritsFrom);

  /**
   * Gets the superimposition elements.
   * @return the list of superimposition elements.
   */
  public List getSuperimposesOn();

  /**
   * Adds a superimposition element to this concept.
   * @param superimposesOn the superimposition element to add.
   */
  public void addSuperimposesOn(RefinedElement superimposesOn);

  /**
   * Adds a superimposition element to this concept.
   * @param index the index at which to add the element.
   * @param superimposesOn the superimposition element to add.
   */
  public void addSuperimposesOn(int index, RefinedElement superimposesOn);

  /**
   * Removes a superimposition element from this concept.
   * @param superimposesOn the superimposition element to remove.
   */
  public void removeSuperimposesOn(RefinedElement superimposesOn);
  
  /**
   * Gets the superimposing elements.
   * @return the list of superimposing elements.
   */
  public List getSuperimposers();

  /**
   * Adds a superimposing element to this concept.
   * @param superimposer the superimposing element to add.
   */
  public void addSuperimposer(RefinedElement superimposer);

  /**
   * Adds a superimposing element to this concept.
   * @param index the index at which to add the element.
   * @param superimposer the superimposing element to add.
   */
  public void addSuperimposer(int index, RefinedElement superimposer);

  /**
   * Removes a superimposing element from this concept.
   * @param superimposer the superimposing element to remove.
   */
  public void removeSuperimposer(RefinedElement superimposer);

  /**
   * Gets the icon image URI.
   * @return the icon URI.
   */
  public String getIconUri();

  /**
   * Gets the icon image URI.
   * @param uri the icon URI.
   */
  public void setIconUri(String uri);

  /**
   * Gets the model of which this element is the default element, if any.
   * @return the model of which this element is the default element, if any.
   */
  public Model getDefaultElementOf();
  
  /**
   * Sets the model of which this element is the default element, if any.
   * @param model the model of which this element is the default element, if any.
   */
  public void setDefaultElementOf(Model model);
}
