/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.lang.impl;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import be.ac.vub.cocompose.lang.Element;

/**
 * Default implementation for Element
 * @author Dennis Wagelaar
 * @version 0.2.0, 02/10/2003
 * @since 0.2.0
 */
public class DefaultElement implements Element
{
  transient private PropertyChangeSupport listeners = new PropertyChangeSupport(this);

  /**
   * @see be.ac.vub.cocompose.lang.Element#addPropertyChangeListener(java.beans.PropertyChangeListener)
   */
  public void addPropertyChangeListener(PropertyChangeListener listener)
  {
    listeners.addPropertyChangeListener(listener);
  }

  /**
   * @see be.ac.vub.cocompose.lang.Element#removePropertyChangeListener(java.beans.PropertyChangeListener)
   */
  public void removePropertyChangeListener(PropertyChangeListener listener)
  {
    listeners.removePropertyChangeListener(listener);
  }

  /**
   * Notifies listeners of a property change.
   * @param id - the property id.
   * @param oldValue - the old value.
   * @param newValue - the new value.
   */
  protected void firePropertyChange(String id, Object oldValue, Object newValue)
  {
    listeners.firePropertyChange(id, oldValue, newValue);
  }

  /**
   * Notifies listeners of a structure change.
   * @param id - the property id.
   * @param child - the newly added/removed child.
   */
  protected void fireStructureChange(String id, Object child)
  {
    listeners.firePropertyChange(id, null, child);
  }
}
