/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.log;

/**
 * Logging functionality interface.
 * @author Dennis Wagelaar
 * @version 0.2.8, 19/04/2004
 * @since 0.2.7
 */
public interface Log
{
  /**
   * Status severity constant (value 0) indicating this status represents the
   * nominal case. This constant is also used as the status code representing 
   * the nominal case.
   */
  public static final int OK = 0;
  
  /**
   * Status type severity (bit mask, value 1) indicating this status is 
   * informational only.
   */
  public static final int INFO = 1;

  /**
   * Status type severity (bit mask, value 2) indicating this status represents 
   * a warning.
   */
  public static final int WARNING = 2;

  /**
   * Status type severity (bit mask, value 4) indicating this status represents 
   * an error.
   */
  public static final int ERROR = 4;
  
  /**
   * Logs a message. Following messages will be logged as a
   * sub-message of this message, until endLog() is called.
   * @param message the log message.
   * @param level the log level (OK, INFO, WARNING, ERROR)
   * @param exception the related exception, if any.
   */
  public void log(String message, int level, Throwable exception);
  
  /**
   * End marker for the last log message. Following log messages
   * will be logged independantly.
   * @param level the log level (OK, INFO, WARNING, ERROR)
   */
  public void endLog(int level);
  
  /**
   * Reports an exception/error in the log and on the screen.
   * @param e the exception to report.
   */
  public void report(Throwable e);

  /**
   * Returns true if debugging messages (OK, INFO) should be logged.
   * @return true if debugging messages should be logged.
   */
  public boolean isDebugging();

  /**
   * Sets debugging message logging (OK, INFO).
   * @param b if true, log debug messages.
   */
  public void setDebugging(boolean b);
}
