/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: ApplySolutionPattern.java,v 1.2 2004/05/18 11:40:13 dwagelaa Exp $
 * $Log: ApplySolutionPattern.java,v $
 * Revision 1.2  2004/05/18 11:40:13  dwagelaa
 * 0 worked on EPS and SVG export: both work now, except that images aren't supported in SVG
 * 0 worked on ApplySolutionPattern: the result is now properly displayed in the editor, and undoable ;-)
 *
 * Revision 1.1  2004/05/17 14:02:54  dwagelaa
 * + added initial ApplySolutionPattern functionality
 * 0 changed EPSGraphics to AWTGraphics
 * + added initial SVG export functionality (EPS works by now)
 * 0 updated version numbers to 0.2.11
 *
 */

package be.ac.vub.cocompose.operations;

import java.util.List;

import be.ac.vub.cocompose.lang.DefaultTraversingModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.core.SolutionPattern;

/**
 * Applies a solution pattern. Uses {@link LinkSolutionPattern} to
 * create links after all elements have been created. 
 * @author Dennis Wagelaar
 * @version 0.2.11, 17/05/2004
 * @since 0.2.11
 */
public class ApplySolutionPattern extends DefaultTraversingModelVisitor
{
  private ModelFactory factory = null;
  private SolutionPattern solutionPattern = null;
  private RefinedElement patternOwner = null;
  private Model targetModel = null;
  private CopyElement copy = null;

  /**
   * Creates a new ApplySolutionPattern operation.
   * @param factory the factory to create new elements.
   */
  public ApplySolutionPattern(ModelFactory factory)
  {
    if (factory == null)
      throw new Error("factory may not be null");
    this.factory = factory;
    this.copy = new CopyElement(factory);
  }
  
  /**
   * Applies the given solution pattern.
   * @param pattern the solution pattern to apply.
   * @throws ModelElementException if the solution pattern cannot be applied.
   */
  public void apply(SolutionPattern pattern) throws ModelElementException
  {
    if (pattern == null)
      throw new ModelElementException("Solution pattern may not be null");
    this.solutionPattern = pattern;
    this.patternOwner = pattern.getRefinedElement();
    if (patternOwner == null)
      throw new ModelElementException("Solution pattern owner may not be null", solutionPattern);
    this.targetModel = patternOwner.getModel();
    if (targetModel == null)
      throw new ModelElementException("Solution pattern owner must reside inside a model", patternOwner);
    if (solutionPattern.getSolutionModel() == null)
      throw new ModelElementException("Solution pattern must point to a model", solutionPattern);
    visit(solutionPattern.getSolutionModel());
  }
  
  /**
   * Inserts the given concept into the target model.
   * @see be.ac.vub.cocompose.lang.ModelVisitor#visitConcept(be.ac.vub.cocompose.lang.core.Concept)
   */
  public void visitConcept(Concept concept) throws ModelElementException
  {
    boolean defaultElt = (concept.getDefaultElementOf() != null);
    List partOf = concept.getPartOf();
    if (partOf.isEmpty())
    { // default part
      Concept conceptCopy = (Concept) copy.copy(concept);
      copy.reset();
      conceptCopy.setId(null);
      List ownedElts = conceptCopy.getDeepOwnedElements(ModelElement.class);
      for (int index = 0; index < ownedElts.size(); index++)
      {
        ModelElement elt = (ModelElement) ownedElts.get(index);
        elt.setId(null);
        if (defaultElt)
          patternOwner.addOwnedElement(elt);
      }
      if (!defaultElt)
      {
        targetModel.addOwnedElement(conceptCopy);
      }
    }
  }
}
